//===================================================================================================
// TMSCommander
//
// Author:		deangelj
// Date:		June 2010
//
// TMSCommander is a TAP for the Topfield TMS machines (the HD boxes that run TAPs). It's main
// purpose is to provide a single entry point into running TAPs' menus rather than each TAP have to
// use separate keys to configure direct entry and then we have clashes or start order issues.
//
// This TAP makes extensive use of Firebird's library (libFirebird) and so many thanks to him for 
// making it available.
//===================================================================================================

#include                "tap.h"
#include                "libFireBird.h"
#include				"TMSCommander.h"
#include				"settings.h"
#include				"keys.h"
#include				"status_bad.gd"

#define PROGRAM_NAME    "TMSCommander"
#define VERSION         "V1.1"
#define LOGFILE         PROGRAM_NAME ".log"
#define LOGDIR          "/ProgramFiles/Settings/"PROGRAM_NAME"/Logs"
#define LNGFILENAME     PROGRAM_NAME ".lng"

TAP_ID                  (TMSCMDRTAPID);
TAP_PROGRAM_NAME        (PROGRAM_NAME" "VERSION);
TAP_AUTHOR_NAME         ("deangelj");
TAP_DESCRIPTION         ("Manage TMS TAPs, e.g. Menus");
TAP_ETCINFO             (__DATE__);

//--------------------------------
// Language support
typedef enum
{
  STRING_FirmwareNotSupported,
  STRING_MainMenu,
  STRING_FlagMenu,
  STRING_FlagStop,
  STRING_ExitNotAllowed,
  STRING_Exit,
  STRING_TapFailedToShowMenu,
  STRING_TapManageNotSupported,
  STRING_TapFailedToStop,
  STRING_TapStopping,
  STRING_TMSCDRStopping,
  STRING_StopNotSupported,
  STRING_TMSCDRMenu,
  STRING_TMSCommanderKey,
  STRING_MenuKey,
  STRING_StopTMSCDR,
  STRING_ExitButton,
  STRING_Select,
  STRING_TAPMenu,
  STRING_MenuKeyDisplay,
  STRING_ExitMenu,
  STRING_Refresh,
  STRING_EnDisable,
  STRING_MoreInfoDescr,
  STRING_MoreInfoCapabilities,
  STRING_MoreInfoIDs,
  STRING_MoreInfoAuthor,
  STRING_MoreInfoUnknown,
  STRING_FlagUserEvent,
  STRING_UserEvent,
  STRING_TapFailedUserEvent,

  STRING_NRStrings
} TEXTSTRINGS;



//--------------------------------
// This section is used to determine which info screen is in use
typedef enum
{
	SCR_Desc,
	SCR_Capabilities,
	SCR_TAPIDs,
	SCR_Author,

	SCR_ScreenCount
} tScreens;

tScreens	CurrentScreen = SCR_Desc;   //Current info screen


//--------------------------------
// This section is used to determine which state we're in
typedef enum
{
  ST_Inactive,
  ST_LoadTAPtable,
  //ST_InfoThenTable,
  ST_TAPtable,
  ST_LoadMenu,
  ST_Menu,
  ST_Exit
} tState;

tState                  State = ST_Inactive;  //start out without the table on screen

//--------------------------------
// This section is used to store our TAP info that's on screen
typedef struct
{
	tTAPInfo				TapInfo;
	tTMSCMDR_Capabilities	Capabilities;
	bool			Disabled;
} tItem;

tItem	TAPItems[TAP_MAX];   //save a list of the TAP items to make it quicker to go between info
byte	TAPCount=0;

//--------------------------------------
extern dword TMSCMDRKEY;
extern dword MENUKEY;

//use firebird's buttons
//extern TYPE_GrData _Button_red_Gd;
//extern TYPE_GrData _Button_green_Gd;
extern void ChangeDirProgramSettings(char *PName);
static dword stop_handler( dword param1 );

//-------------------------------------- DEBUGGING
char      LogPuffer[256];
//#define DEBUG(...)      {ChangeDirProgramSettings(PROGRAM_NAME); sprintf(LogPuffer, __VA_ARGS__); LogEntry(LOGFILE, PROGRAM_NAME, TRUE, TIMESTAMP_HMS, LogPuffer);}
#define DEBUG //

//===================================================================================================
void WriteLog(char *s)
{
	static bool           FirstCall = TRUE;

	HDD_TAP_PushDir();
	if(FirstCall)
	{
		ChangeDirProgramSettings(PROGRAM_NAME);
		TAP_Hdd_Create("Logs", ATTR_FOLDER);
		FirstCall = FALSE;
	}
	TAP_Hdd_ChangeDir(LOGDIR);
	LogEntry(LOGFILE, PROGRAM_NAME, TRUE, TIMESTAMP_YMDHM, s);
	HDD_TAP_PopDir();
}

//===================================================================================================
dword SendCommandToTAP(dword ID, dword cmd)
{
	return HDD_TAP_SendEvent (ID, FALSE, EVT_TMSCommander, cmd, 0);  //libFirebird routine
}

//===================================================================================================
dword TAPGetCapabilities(dword ID)
{
	return SendCommandToTAP (ID, (dword)(TMSCMDR_Capabilities));
}

//===================================================================================================
bool TAPHasCapability(dword ID, dword capability)
{
	dword status = TAPGetCapabilities(ID);
	DEBUG("Check that tap has capability: %x, status = %x", capability, status);
	return ((status & capability) == capability);
}

//===================================================================================================
bool CurrentItemHasCapability(dword capability)
{
	return TAPHasCapability(OSDMenuItemGetCurrentID(), capability);
}

//===================================================================================================
// ExitTAP
//
// First check that the infobox (telling the user we're quitting) has timed out - otherwise go back to the idle loop
// Tell each running TAP we're exiting before actually quitting
//===================================================================================================

static void ExitTAP()
{
	int					  i;

	//Tell TAPs we're exiting...
	for (i = 0; i < TAPCount; i++)  //16 max
	{
		if (TAPItems[i].TapInfo.TAPID != TMSCMDRTAPID)
			SendCommandToTAP (TAPItems[i].TapInfo.TAPID, (dword)(TMSCMDR_Exiting));
	}

	OSDMenuDestroy();
	TAP_Exit();
}

//===================================================================================================
// ShowMenu
//
// Shows TMSCommander config menu
//===================================================================================================
static void ShowMenu()
{
	OSDMenuDestroy();
	//void OSDMenuInitialize(bool AllowScrollingOfLongText, bool HasValueColumn, bool NumberedItems, bool ScrollLoop, char *TitleLeft, char *TitleRight)
	OSDMenuInitialize(FALSE, TRUE, FALSE, TRUE, PROGRAM_NAME, LangGetStringDefault(STRING_TMSCDRMenu, "TMSCommander Menu"));
	//bool OSDMenuItemAdd(char *Name, char *Value, TYPE_GrData *pNameIconGd, TYPE_GrData *pValueIconGd, bool Selectable, bool ValueArrows, dword ID)
	OSDMenuItemAdd(LangGetStringDefault(STRING_TMSCommanderKey, "TMSCommander Key"), (char *)GetKeyName(TMSCMDRKEY), NULL, NULL, TRUE, TRUE, 0);
	OSDMenuItemAdd(LangGetStringDefault(STRING_MenuKey, "Menu Key"), (char *)GetKeyName(MENUKEY), NULL, NULL, TRUE, TRUE, 0);
	OSDMenuItemAdd(LangGetStringDefault(STRING_StopTMSCDR, "Stop TMSCommander"), NULL, NULL, NULL, TRUE, FALSE, 0);
	OSDMenuButtonAdd(1, BI_Exit, NULL, LangGetStringDefault(STRING_ExitButton, "Exit"));
	OSDMenuButtonAdd(1, BI_Ok, NULL, LangGetStringDefault(STRING_Select, "Select"));
	OSDMenuUpdate(FALSE);
	State = ST_Menu;
}

//============================================================================================================
// UpdateMenuButtons
//
//===================================================================================================
void UpdateMenuButtons(void)
{
	dword                 CurrentCapabilities;

	OSDMenuButtonsClear();

	CurrentCapabilities = TAPItems[OSDMenuGetCurrentItem()].Capabilities & 0xffff;

	if(CurrentCapabilities & TMSCMDR_HaveMenu)
	{
		DEBUG("MenuKey=%s\n", MENUKEY==RKEY_Menu?"Menu":"OK");
		OSDMenuButtonAdd(1, MENUKEY==RKEY_Menu?BI_Menu:BI_Ok,  NULL, LangGetStringDefault(STRING_TAPMenu, " TAP Menu"));
	}

	if(CurrentCapabilities & TMSCMDR_CanBeStopped)
		OSDMenuButtonAdd(1, BI_Red,   NULL, LangGetStringDefault(STRING_MenuKeyDisplay, " Stop TAP"));

	if(CurrentCapabilities & TMSCMDR_HaveUserEvent)
		OSDMenuButtonAdd(1, BI_Yellow,   NULL, LangGetStringDefault(STRING_UserEvent, " UserEvent"));

	OSDMenuButtonAdd(2, BI_Exit,  NULL,  LangGetStringDefault(STRING_ExitMenu, " Exit Menu"));
	OSDMenuButtonAdd(2, BI_Green, NULL,  LangGetStringDefault(STRING_Refresh, " Refresh"));
	OSDMenuButtonAdd(2, BI_Blue , NULL,  LangGetStringDefault(STRING_EnDisable, " En-/Disable"));

	switch (CurrentScreen)
	{
	case SCR_Desc:
		OSDMenuButtonAdd(3, BI_Info, NULL,  LangGetStringDefault(STRING_MoreInfoDescr, " More info (Currently showing TAP Description)"));
		break;
	case SCR_Capabilities:
		OSDMenuButtonAdd(3, BI_Info, NULL,  LangGetStringDefault(STRING_MoreInfoCapabilities, " More info (Currently showing TAP Capabilities)"));
		break;
	case SCR_TAPIDs:
		OSDMenuButtonAdd(3, BI_Info, NULL,  LangGetStringDefault(STRING_MoreInfoIDs, " More info (Currently showing TAP IDs)"));
		break;
	case SCR_Author:
		OSDMenuButtonAdd(3, BI_Info, NULL,  LangGetStringDefault(STRING_MoreInfoAuthor, " More info (Currently showing TAP Author)"));
		break;
	default:
		OSDMenuButtonAdd(3, BI_Info, NULL,  LangGetStringDefault(STRING_MoreInfoUnknown, " More info (??)"));
		break;
	}
}


//===================================================================================================
// LoadTAPtable 
//
// This is the main screen for the TAP. It shows a row for every TAP started on the machine. 
// Only those that acknowledge at least one capability will be selectable.
//===================================================================================================
static void LoadTAPtable()
{

	tTAPInfo             *pTAPInfo;
	tTMSTAPTaskTable     *TMSTAPTaskTable;
	char                  AbsFileName[100];
	int i;
	static bool			  Selected;

	TMSTAPTaskTable = (tTMSTAPTaskTable*)FIS_vTAPTable();
	if(!TMSTAPTaskTable)
	{
		//not supported on this firmware
		OSDMenuInfoBoxShow(PROGRAM_NAME, LangGetStringDefault(STRING_FirmwareNotSupported, "This machine's firmware does not support this software"), 300);
		LangUnloadStrings();
		TAP_Exit();
		return;
	}

	OSDMenuDestroy();
	OSDMenuInitialize(TRUE, TRUE, FALSE, TRUE, PROGRAM_NAME, LangGetStringDefault(STRING_MainMenu, "Main Menu"));
	//OSDMenuLogo(493, 486, &_qplogoGd);

	OSDMenuItemModifyValueXPos(300);  //adjust the value column start xpos

	//get a list of the running taps, send each a message to see if they can be managed
	Selected = FALSE; TAPCount = 0;

	for (i = 0; i < TAP_MAX; i++)  //0..15
	{
		if(TMSTAPTaskTable[i].file)   //needed to get tap info
		{
			if(HDD_GetAbsolutePathByTypeFile((TYPE_File*)TMSTAPTaskTable[i].file, AbsFileName))  //Look up TAP file to get more details
			{
				pTAPInfo = &TAPItems[TAPCount].TapInfo;
				if(HDD_TAP_GetInfoByAbsPath(AbsFileName, pTAPInfo))
				{
					if((TAPItems[TAPCount].Capabilities = TAPGetCapabilities(pTAPInfo->TAPID)) < 0x1000000)
						TAPItems[TAPCount].Capabilities = 0;
					DEBUG("TAP %s, Capabilities: %x", pTAPInfo->TAPName, TAPItems[TAPCount].Capabilities);

					OSDMenuItemAdd(pTAPInfo->TAPName, pTAPInfo->Description, NULL, NULL, TRUE, FALSE, pTAPInfo->TAPID);
					if (!Selected)
					{
						OSDMenuSelectItem(i);
						Selected = TRUE;
					}

					TAPItems[TAPCount].Disabled = HDD_TAP_isDisabled(pTAPInfo->TAPID);
					if(TAPItems[TAPCount].Disabled)
						OSDMenuItemModifyNameIcon(TAPCount, &_status_bad_Gd);

					TAPCount++;  //have a valid TAP detail block
				}
				else
				{
					DEBUG("  failed to read the TAP details\n");
				}
			}
		}
	}

	UpdateMenuButtons();
	OSDMenuUpdate(FALSE);

	State = ST_TAPtable;

}

//===================================================================================================
// ToggleEventSink
//
//===================================================================================================
void ToggleEventSink(void)
{
	bool                 isDisabled;

	isDisabled = HDD_TAP_isDisabled(OSDMenuItemGetCurrentID());

	if(HDD_TAP_Disable(OSDMenuItemGetCurrentID(), !isDisabled))
	{
		OSDMenuItemModifyNameIcon(OSDMenuGetCurrentItem(), !isDisabled ? &_status_bad_Gd : NULL);
		OSDMenuUpdate(FALSE);

		TAPItems[OSDMenuGetCurrentItem()].Disabled = !isDisabled;
	}
}

//===================================================================================================
// ShowNextInfoScreen
//
// When user presses "i" on TAP table screen cycle through info screens
//===================================================================================================
static void ShowNextInfoScreen()
{
	int i;
	char value[128];

	if (++CurrentScreen == SCR_ScreenCount)
		CurrentScreen = SCR_Desc;


	for (i = 0; i < TAPCount; i++)  //16 max
	{
		switch (CurrentScreen)
		{
		case SCR_Desc:
			OSDMenuItemModifyValue(i, TAPItems[i].TapInfo.Description);
			break;
		case SCR_Capabilities:
			TAP_SPrint(value, "%s%s%s", 
				((TAPItems[i].Capabilities & TMSCMDR_HaveMenu) == TMSCMDR_HaveMenu)?LangGetStringDefault(STRING_FlagMenu, "MENU "):"",
				((TAPItems[i].Capabilities & TMSCMDR_HaveUserEvent) == TMSCMDR_HaveUserEvent)?LangGetStringDefault(STRING_FlagUserEvent, " USEREVENT "):"",
				((TAPItems[i].Capabilities & TMSCMDR_CanBeStopped) == TMSCMDR_CanBeStopped)?LangGetStringDefault(STRING_FlagStop, "STOP"):""	);
			OSDMenuItemModifyValue(i, value);
			break;
		case SCR_TAPIDs:
			TAP_SPrint(value, "0x%0x", TAPItems[i].TapInfo.TAPID);
			OSDMenuItemModifyValue(i, value);
			break;
		case SCR_Author:
			TAP_SPrint(value, "%s", TAPItems[i].TapInfo.Author);
			OSDMenuItemModifyValue(i, value);
			break;
		default:
			break;
		}
	}

	UpdateMenuButtons();	
	OSDMenuUpdate(FALSE);

}

//===================================================================================================
// key_handler for menu atate
//
// When in the config menu, this section handles the keys that are pressed
//===================================================================================================

static dword key_handler_menu( dword param1 )
{
	int item = OSDMenuGetCurrentItem();

	DEBUG("Key handler - menu state");
	//Accept Left/Right for value changes, OK to select, Exit to Exit TAP

	if (param1 == RKEY_Ok && item != 2) param1 = RKEY_Right;

	switch (param1)
	{
		case RKEY_Exit:
			SaveSettings(PROGRAM_NAME);
			OSDMenuDestroy();
			State = ST_LoadTAPtable;
			break;
		case RKEY_Ok:
		{
			bool OKtoExit = TRUE;
			int  i;

			//NEVER stop if a TAP has been disabled. If a TAP has been disabled by this TAP
			//  and this TAP exits, a crash will occur!!!
			for(i = 0; i < TAPCount; i++)
				if(TAPItems[i].Disabled)
				{
					OSDMenuInfoBoxShow(PROGRAM_NAME, LangGetStringDefault(STRING_ExitNotAllowed, "Exiting TMSCommander is not allowed\nuntil all TAPs are enabled"), 300);
					OKtoExit = FALSE;
					break;
				}

			if(OKtoExit)
			{
				OSDMenuInfoBoxShow(PROGRAM_NAME, LangGetStringDefault(STRING_Exit, "TMSCommander is exiting"), 300);
				State = ST_Exit;
			}
			break;
		}
		case RKEY_Left:
			item = OSDMenuGetCurrentItem();
			switch (item)
			{
			case 0:
				TMSCMDRKEY = GetPriorKey(TMSCMDRKEY);
				OSDMenuItemModifyValue(item, (char *)GetKeyName(TMSCMDRKEY));
				break;
			case 1:
				MENUKEY = MENUKEY==RKEY_Ok?RKEY_Menu:RKEY_Ok;
				OSDMenuItemModifyValue(item, (char *)GetKeyName(MENUKEY));
				break;
			default:
				break;
			}
			OSDMenuUpdate(FALSE);
			break;
		case RKEY_Right:
			item = OSDMenuGetCurrentItem();
			switch (item)
			{
			case 0:
				TMSCMDRKEY = GetNextKey(TMSCMDRKEY);
				OSDMenuItemModifyValue(item, (char *)GetKeyName(TMSCMDRKEY));
				break;
			case 1:
				MENUKEY = MENUKEY==RKEY_Ok?RKEY_Menu:RKEY_Ok;
				OSDMenuItemModifyValue(item, (char *)GetKeyName(MENUKEY));
				break;
			default:
				break;
			}
			OSDMenuUpdate(FALSE);
			break;
		default:
			break;
	}
	return 0;

}

//===================================================================================================
// key_handler for taptable atate
//
// When on the TAP screen, this section handles any key presses
//===================================================================================================

static dword key_handler_taptable( dword param1 )
{
	dword status;
	dword ID = OSDMenuItemGetCurrentID();

	DEBUG("Key handler - taptable state");
	//OK to request TAP to show menu, and TMSCMDRKEY to go to inactive is all we support at the moment

	switch (param1)
	{
	case RKEY_Ok:			//call TAP to show menu
		if (MENUKEY!=RKEY_Ok) break;
		//allow to fall through
	case RKEY_Menu:			//call TAP to show menu
		if (param1==RKEY_Menu && MENUKEY!=RKEY_Menu) break;

		//does the TAP support being managed
		if (CurrentItemHasCapability(TMSCMDR_HaveMenu))
		{
			//Tell the remote TAP to show its menu
			OSDMenuDestroy();
			status = SendCommandToTAP (ID, (dword)(TMSCMDR_Menu));
			if (status != TMSCMDR_OK)
			{
				//tell user we weren't able to tell the tap to show its menu
				OSDMenuInfoBoxShow(PROGRAM_NAME, LangGetStringDefault(STRING_TapFailedToShowMenu, "TAP did not respond correctly to Show Menu command"), 300);
			}
			else if (ID != TMSCMDRTAPID) //not this TAP, so wait for the show menu key
			{
				State = ST_Inactive;
			}
		}
		else
		{
			OSDMenuInfoBoxShow(PROGRAM_NAME, LangGetStringDefault(STRING_TapManageNotSupported, "TAP does not support being managed"), 300);
		}
		break;
	case RKEY_NewF1:			//Stop TAP
		if (CurrentItemHasCapability(TMSCMDR_CanBeStopped))
		{
			if (ID != TMSCMDRTAPID) //not this TAP
			{
				status = SendCommandToTAP (ID, (dword)(TMSCMDR_Stop));
				if (status != TMSCMDR_OK)
				{
					OSDMenuInfoBoxShow(PROGRAM_NAME, LangGetStringDefault(STRING_TapFailedToStop, "TAP did not respond correctly to Stop command"), 300);
				}
				else
				{
					//OSDMenuInfoBoxShow(PROGRAM_NAME, "TAP is stopping", 300);
					//State = ST_InfoThenTable;
					OSDMenuInfoBoxShow(PROGRAM_NAME, LangGetStringDefault(STRING_TapStopping, "TAP is stopping"), 300);
					State = ST_LoadTAPtable;
				}
			}
			else  //stopping this TAP
			{
				stop_handler(0);
				OSDMenuInfoBoxShow(PROGRAM_NAME, LangGetStringDefault(STRING_TMSCDRStopping, "TMSCommander is exiting"), 300);
				State = ST_Exit;
			}
		}
		else
		{
			OSDMenuInfoBoxShow(PROGRAM_NAME, LangGetStringDefault(STRING_StopNotSupported, "TAP does not support being stopped"), 300);
		}
		break;
	case RKEY_Exit:				//Exit TAP table
		OSDMenuDestroy();
		State = ST_Inactive;
		break;
	case RKEY_F2:		//refresh
		LoadTAPtable();
		break;
	case RKEY_F3:		//userevent
		//does the TAP support being managed
		if (CurrentItemHasCapability(TMSCMDR_HaveUserEvent))
		{
			OSDMenuDestroy();
			status = SendCommandToTAP (ID, (dword)(TMSCMDR_UserEvent));
			if (status != TMSCMDR_OK)
			{
				//tell user we weren't able to tell the tap to show its menu
				OSDMenuInfoBoxShow(PROGRAM_NAME, LangGetStringDefault(STRING_TapFailedUserEvent, "TAP did not respond correctly to UserEvent command"), 300);
			}
			else
			{
				State = ST_Inactive;
			}
		}
		else
		{
			OSDMenuInfoBoxShow(PROGRAM_NAME, LangGetStringDefault(STRING_TapManageNotSupported, "TAP does not support being managed"), 300);
		}
		break;
	case RKEY_Info:		//show next info screen
		ShowNextInfoScreen();
		break;
	case RKEY_F4:     //En-/disable the TAP_EventHandler
		ToggleEventSink();
		break;
	default:
		break;
	
	}
	return 0;

}

//===================================================================================================
// key_handler for inactive atate
//===================================================================================================
/*
static dword key_handler_inactive( dword param1 )
{
	//only respond on the TMSCMDRKEY
	DEBUG("Key handler - inactive state");
	if(param1 == TMSCMDRKEY)
	{
		State = ST_LoadTAPtable;
		param1 = 0;
	}
	return param1;
}
*/

//===================================================================================================
// key_handler - 
//===================================================================================================

static dword key_handler( dword param1, dword subState )
{
	if (subState == SUBSTATE_Normal && param1 == TMSCMDRKEY)
	{
		State = ST_LoadTAPtable;
		param1 = 0;
	}
	else if (subState == SUBSTATE_MainMenu)
	{
		if (State == ST_TAPtable)	return key_handler_taptable( param1 );
		if (State == ST_Menu)		return key_handler_menu( param1 );
	}

	return param1;
}

//===================================================================================================
// idle_handler - 
//
// When idle figure out what state we're in 
//===================================================================================================

static dword idle_handler( dword param1 )
{
	switch (State)
	{
		case ST_LoadTAPtable:			LoadTAPtable(); break;
		//case ST_InfoThenTable:			InfoThenTable(); break;
		case ST_LoadMenu:				ShowMenu(); break;
		case ST_Exit:					ExitTAP(); break;
		default:						break;
	}
	return param1;
}

//===================================================================================================
// stop_handler -
//
// Received a stop event. Make sure all TAPs are enabled
//===================================================================================================

static dword stop_handler( dword param1 )
{
	//bool								isDisabled;
	int									i;
	tTMSTAPTaskTable		*TMSTAPTaskTable;
	tTAPInfo						pTAPInfo;
	char								AbsFileName[100];

	TMSTAPTaskTable = (tTMSTAPTaskTable*)FIS_vTAPTable();
	if(!TMSTAPTaskTable) return param1;

	for (i = 0; i < TAP_MAX; i++)  //0..15
	{
		if((TMSTAPTaskTable[i].Status == 1) && TMSTAPTaskTable[i].file)
		{
			if(HDD_GetAbsolutePathByTypeFile((TYPE_File*)TMSTAPTaskTable[i].file, AbsFileName) && HDD_TAP_GetInfoByAbsPath(AbsFileName, &pTAPInfo))
			{
				if(HDD_TAP_isDisabled(pTAPInfo.TAPID))
					HDD_TAP_Disable(pTAPInfo.TAPID, FALSE);
			}
		}
	}

	return param1;
}

//===================================================================================================
// TMSCommander_handler - 
//
// This is the handler for when I get called by the menu
//===================================================================================================

static dword TMSCommander_handler( dword param1 )
{

	DEBUG("TMSCommandler_handler...key: %d\n", param1);

	switch (param1)
	{
	   case TMSCMDR_Capabilities:
			DEBUG("Got request for TMSCMDR_Capabilities\n");
			return (dword)(TMSCMDR_HaveMenu | TMSCMDR_CanBeStopped);
	   case TMSCMDR_Stop:
			//call your TAP's exit function or
			ExitTAP();
			return TMSCMDR_OK;
	   case TMSCMDR_Menu:
			//call your TAP's menu
			DEBUG("Got request for TMSCMDR_Menu\n");
			State = ST_LoadMenu;
			return TMSCMDR_OK;
	   case TMSCMDR_IsAlive:
			//TAP wants to know if we're alive
			DEBUG("Got request for TMSCMDR_IsAlive\n");
			return TMSCMDR_OK;
	   default:
			//unknown TMSCommander function
			DEBUG("Got request for unknown function\n");
			return TMSCMDR_UnknownFunction;
	}
}


//===================================================================================================
// TAP_EventHsndler
//
// The heart of the TAP - it needs to keep beating... 
// 
//===================================================================================================
dword TAP_EventHandler(word event, dword param1, dword param2)
{
	(void) param2;
    dword mainState, subState;

	if(event == EVT_STOP) 
		return stop_handler( param1 );

	// Get State Information from TAP interface
    TAP_GetState( &mainState, &subState );
 
    //if ((mainState != STATE_Normal) || (subState != SUBSTATE_Normal))
    if (mainState != STATE_Normal) 
        return param1;

	//DEBUG("Substate: %d", subState);

	//Substate == SUBSTATE_Normal is when no menu is being displayed by me, SUBSTATE_MainMenu when the menu is dislayed

	//Absorb all events if any of the dialogs are visible
	if(OSDMenuInfoBoxIsVisible() || OSDMenuMessageBoxIsVisible() || OSDMenuColorPickerIsVisible())
	{
		OSDMenuEvent(&event, &param1, &param2);
		return 0;
	}

 	// if OSD key event has been handled just exit (param1 is 0)
	if ((subState == SUBSTATE_MainMenu) && OSDMenuEvent(&event, &param1, &param2))
	{
		//only update for the Main TAP menu not for the config menu
		if (State != ST_Menu)
			UpdateMenuButtons();

		OSDMenuUpdate(FALSE);
		return param1;
	}

	// Pass event to appropriate handler
	switch( event )
	{
		case EVT_KEY: 			return key_handler	( param1, subState );  break;	
		case EVT_IDLE:			return idle_handler ( param1 );  break;
		case EVT_TMSCommander:	return TMSCommander_handler( param1 ); break;
		case EVT_UART:	
		case EVT_RBACK:
		case EVT_SVCSTATUS:
		case EVT_DEMUX:
		case EVT_STOP:			//Stop has been handled at the beginning of the event handler
		case EVT_VIDEO_CONV:	return param1;  break;	

		default:		DEBUG("Invalid event found: %d\n", event); 
			break;
	}
	return param1;

}

//===================================================================================================
// TAP_Main
//
// where life begins... - just Load our ini file and stay resident waiting for the show menu key
//===================================================================================================
int TAP_Main(void)
{
	//DumpTaps();
	//return 0;

	//Don't even start without a language file
	if (!LangLoadStrings(LNGFILENAME, STRING_NRStrings, LAN_English, PROGRAM_NAME))
	{
		char  s[50];

		TAP_SPrint (s, "Language file '%s' not found!", LNGFILENAME);
		OSDMenuInfoBoxShow(PROGRAM_NAME, s, 300);
		do
		{
			OSDMenuEvent(NULL, NULL, NULL);
		} while(OSDMenuInfoBoxIsVisible());

		//return 0;   support no language file
	}

	LoadSettings(PROGRAM_NAME);
	return 1;

}
